from pm4pymdl import algo, objects, visualization, order_log_generation

__version__ = '0.0.6'
__doc__ = "Process Mining for Python - Multi-Dimensional Event Logs"
__author__ = 'PADS'
__author_email__ = 'a.berti@pads.rwth-aachen.de'
__maintainer__ = 'PADS'
__maintainer_email__ = "a.berti@pads.rwth-aachen.de"
